@@ -3,12 +3,12 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef CONF_H
@@ -43,30 +42,33 @@
  * An account
  */
 
-#define ACC_HOST		(1 << 0)
-#define ACC_PORT		(1 << 1)
-#define ACC_TIMEOUT		(1 << 2)
-#define ACC_PROTOCOL		(1 << 3)
-#define ACC_AUTO_FROM		(1 << 4)
-#define ACC_FROM		(1 << 5)
-#define ACC_DOMAIN		(1 << 6)
-#define ACC_MAILDOMAIN		(1 << 7)
-#define ACC_DSN_RETURN		(1 << 8)
-#define ACC_DSN_NOTIFY		(1 << 9)
-#define ACC_KEEPBCC		(1 << 10)
-#define ACC_AUTH_MECH		(1 << 11)
-#define ACC_USERNAME		(1 << 12)
-#define ACC_PASSWORD		(1 << 13)
-#define ACC_NTLMDOMAIN		(1 << 14)
-#define ACC_TLS			(1 << 15)
-#define ACC_TLS_NOSTARTTLS	(1 << 16)
-#define ACC_TLS_KEY_FILE	(1 << 17)
-#define ACC_TLS_CERT_FILE	(1 << 18)
-#define ACC_TLS_TRUST_FILE	(1 << 19)
-#define ACC_TLS_NOCERTCHECK	(1 << 20)
-#define ACC_TLS_FORCE_SSLV3	(1 << 21)
-#define ACC_LOGFILE		(1 << 22)
-#define ACC_SYSLOG		(1 << 23)
+#define ACC_HOST			(1 << 0)
+#define ACC_PORT			(1 << 1)
+#define ACC_TIMEOUT			(1 << 2)
+#define ACC_PROTOCOL			(1 << 3)
+#define ACC_AUTO_FROM			(1 << 4)
+#define ACC_FROM			(1 << 5)
+#define ACC_DOMAIN			(1 << 6)
+#define ACC_MAILDOMAIN			(1 << 7)
+#define ACC_DSN_RETURN			(1 << 8)
+#define ACC_DSN_NOTIFY			(1 << 9)
+#define ACC_KEEPBCC			(1 << 10)
+#define ACC_AUTH_MECH			(1 << 11)
+#define ACC_USERNAME			(1 << 12)
+#define ACC_PASSWORD			(1 << 13)
+#define ACC_NTLMDOMAIN			(1 << 14)
+#define ACC_TLS				(1 << 15)
+#define ACC_TLS_NOSTARTTLS		(1 << 16)
+#define ACC_TLS_KEY_FILE		(1 << 17)
+#define ACC_TLS_CERT_FILE		(1 << 18)
+#define ACC_TLS_TRUST_FILE		(1 << 19)
+#define ACC_TLS_CRL_FILE		(1 << 20)
+#define ACC_TLS_NOCERTCHECK		(1 << 21)
+#define ACC_TLS_FORCE_SSLV3		(1 << 22)
+#define ACC_TLS_MIN_DH_PRIME_BITS	(1 << 23)
+#define ACC_TLS_PRIORITIES		(1 << 24)
+#define ACC_LOGFILE			(1 << 25)
+#define ACC_SYSLOG			(1 << 26)
 
 typedef struct
 {
@@ -100,10 +102,13 @@ typedef struct
     char *tls_key_file;		/* file in PEM format */
     char *tls_cert_file;	/* file in PEM format */
     char *tls_trust_file;	/* file in PEM format */
+    char *tls_crl_file;		/* file in PEM format */
     int tls_nostarttls;		/* flag: start TLS immediatly 
 				   (without STARTTLS command)? */
     int tls_nocertcheck;	/* flag: do not check certificate? */
     int tls_force_sslv3;	/* flag: force SSLv3? */
+    int tls_min_dh_prime_bits;	/* parameter; -1 for default */
+    char *tls_priorities;	/* parameter; NULL for default */
     /* logging */
     char *logfile;		/* NULL or logfile */
     char *syslog;		/* NULL or syslog facility */
