@@ -3,12 +3,12 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,11 +40,13 @@
 # include "hmac.h"
 #endif
 
+#include "c-ctype.h"
 #include "gettext.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
 
 #include "list.h"
+#include "readbuf.h"
 #include "net.h"
 #include "smtp.h"
 #include "stream.h"
@@ -107,10 +108,10 @@ smtp_server_t smtp_new(FILE *debug, int protocol)
     smtp_server_t srv;
 
     srv.fd = -1;
-    net_readbuf_init(&(srv.readbuf));
 #ifdef HAVE_TLS
     tls_clear(&srv.tls);
 #endif /* HAVE_TLS */
+    readbuf_init(&(srv.readbuf));
     srv.protocol = protocol;
     srv.cap.flags = 0;
     srv.cap.size = 0;
@@ -166,8 +167,8 @@ int smtp_get_msg(smtp_server_t *srv, list_t **msg, char **errstr)
 #ifdef HAVE_TLS
 	if (tls_is_active(&srv->tls))
 	{
-	    if (tls_gets(&srv->tls, line, SMTP_BUFSIZE, &len, errstr) 
-		    != TLS_EOK)
+	    if (tls_gets(&srv->tls, &(srv->readbuf), 
+			line, SMTP_BUFSIZE, &len, errstr) != TLS_EOK)
 	    {
 		list_xfree(l, free);
 		return SMTP_EIO;
@@ -176,8 +177,8 @@ int smtp_get_msg(smtp_server_t *srv, list_t **msg, char **errstr)
 	else
 	{
 #endif /* HAVE_TLS */
-	    if (net_gets(srv->fd, &(srv->readbuf), line, SMTP_BUFSIZE, &len, 
-			errstr) != NET_EOK)
+	    if (net_gets(srv->fd, &(srv->readbuf), 
+			line, SMTP_BUFSIZE, &len, errstr) != NET_EOK)
 	    {
 		list_xfree(l, free);
 		return SMTP_EIO;
@@ -446,7 +447,7 @@ int smtp_init(smtp_server_t *srv, const char *ehlo_domain, list_t **errmsg,
 	/* make line uppercase */
     	for (i = 4; (size_t)i < len; i++)
 	{
-	    s[i] = toupper((unsigned char)s[i]);
+	    s[i] = c_toupper((unsigned char)s[i]);
 	}
 	/* search capabilities */
 	if (strncmp(s + 4, "STARTTLS", 8) == 0)
@@ -530,11 +531,12 @@ int smtp_init(smtp_server_t *srv, const char *ehlo_domain, list_t **errmsg,
 
 #ifdef HAVE_TLS
 int smtp_tls_init(smtp_server_t *srv, const char *tls_key_file, 
-	const char *tls_ca_file, const char *tls_trust_file, 
-	int force_sslv3, char **errstr)
+	const char *tls_cert_file, const char *tls_trust_file, 
+	const char *tls_crl_file, int force_sslv3, int min_dh_prime_bits,
+	const char *priorities,	char **errstr)
 {
-    return tls_init(&srv->tls, tls_key_file, tls_ca_file, tls_trust_file, 
-	    force_sslv3, errstr);
+    return tls_init(&srv->tls, tls_key_file, tls_cert_file, tls_trust_file,
+	    tls_crl_file, force_sslv3, min_dh_prime_bits, priorities, errstr);
 }
 #endif /* HAVE_TLS */
 
@@ -566,6 +568,11 @@ int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char **errstr)
 	*errstr = xasprintf(_("command %s failed"), "STARTTLS");
 	return SMTP_EPROTO;
     }
+    if (!readbuf_is_empty(&(srv->readbuf)))
+    {
+	*errstr = xasprintf(_("command %s failed"), "STARTTLS");
+	return SMTP_EPROTO;
+    }
     list_xfree(msg, free);
     return SMTP_EOK;
 }
@@ -1401,6 +1408,7 @@ int smtp_send_envelope(smtp_server_t *srv,
     int data_reply_was_rcvd = 0;
     int pipeline_limit = 1;
     int piped_commands = 0;
+    int status;
 
     
     *error_msg = NULL;
@@ -1423,12 +1431,14 @@ int smtp_send_envelope(smtp_server_t *srv,
 		if (dsn_return)
 		{
 		    e = smtp_send_cmd(srv, errstr, "MAIL FROM:<%s> RET=%s", 
-			    envelope_from, dsn_return);
+			    strcasecmp(envelope_from, "MAILER-DAEMON") == 0 
+			    ? "" : envelope_from, dsn_return);
 		}
 		else
 		{
 		    e = smtp_send_cmd(srv, errstr, "MAIL FROM:<%s>", 
-			    envelope_from);
+			    strcasecmp(envelope_from, "MAILER-DAEMON") == 0 
+			    ? "" : envelope_from);
 		}
 		if (e != SMTP_EOK)
 		{
@@ -1473,12 +1483,14 @@ int smtp_send_envelope(smtp_server_t *srv,
 		{
 		    return e;
 		}
-		if (smtp_msg_status(msg) != 250)
+		status = smtp_msg_status(msg);
+		if (status != 250)
 		{
 		    *error_msg = msg;
 		    *errstr = xasprintf(_("envelope from address %s not "
 				"accepted by the server"), envelope_from);
-		    return SMTP_EINVAL;
+		    return (status >= 400 && status <= 499 
+			    ? SMTP_EUNAVAIL : SMTP_EINVAL);
 		}
 		list_xfree(msg, free);
 		mailfrom_reply_was_rcvd = 1;
@@ -1490,13 +1502,15 @@ int smtp_send_envelope(smtp_server_t *srv,
 		{
 		    return e;
 		}
-		if (smtp_msg_status(msg) != 250)
+		status = smtp_msg_status(msg);
+		if (status != 250 && status != 251)
 		{
 		    *error_msg = msg;
 		    *errstr = xasprintf(_("recipient address %s not "
 				"accepted by the server"),
 			    (char *)(rcpt_recv->data));
-		    return SMTP_EINVAL;
+		    return (status >= 400 && status <= 499 
+			    ? SMTP_EUNAVAIL : SMTP_EINVAL);
 		}
 		list_xfree(msg, free);		
 	    }
@@ -1506,12 +1520,14 @@ int smtp_send_envelope(smtp_server_t *srv,
 		{
 		    return e;
 		}
+		status = smtp_msg_status(msg);
 		if (smtp_msg_status(msg) != 354)
 		{
 		    *error_msg = msg;
 		    *errstr = xasprintf(
 			    _("the server does not accept mail data"));
-		    return SMTP_EUNAVAIL;
+		    return (status >= 400 && status <= 499 
+			    ? SMTP_EUNAVAIL : SMTP_EINVAL);
 		}
 		list_xfree(msg, free);
 		data_reply_was_rcvd = 1;
@@ -1684,15 +1700,16 @@ int smtp_end_mail(smtp_server_t *srv, list_t **error_msg, char **errstr)
     {
 	return e;
     }
+    *error_msg = msg;
     if (smtp_msg_status(msg) != 250)
     {
-	*error_msg = msg;
 	*errstr = xasprintf(_("the server did not accept the mail"));
 	return SMTP_EUNAVAIL;
     }
-    list_xfree(msg, free);
-    
-    return SMTP_EOK;
+    else
+    {
+	return SMTP_EOK;
+    }
 }
 
 
